import array
import copy
import click

from cinematinator.offline import Offline
from cinematinator.offlineMFFD import OfflineMFFD
from cinematinator.types import Cinema, SeatStatus


@click.group()
def cli() -> None:
    pass


@click.command()
def offline() -> None:
    Program().offline()


@click.command()
def online() -> None:
    Program().online()


cli.add_command(offline)
cli.add_command(online)


if __name__ == "__main__":
    cli()


class Program:
    def _read_cinema(self) -> Cinema:
        height = int(input())
        width = int(input())
        seats = []
        for _ in range(height):
            row = input()
            for seat in row:
                if seat == "0":
                    seats.append(SeatStatus.NoSeat.value)
                elif seat == "1":
                    seats.append(SeatStatus.Available.value)
        return Cinema(width, height, array.array("b", seats))

    def _print_cinema(self, cinema: Cinema) -> None:
        for y in range(cinema.height):
            for x in range(cinema.width):
                print(
                    {
                        SeatStatus.NoSeat: "0",
                        SeatStatus.Available: "1",
                        SeatStatus.Blocked: "1",
                        SeatStatus.Taken: "x",
                    }[cinema.get_seat(x, y)],
                    end="",
                )
            print()

    def offline(self) -> None:
        cinema = self._read_cinema()
        cinema2 = copy.deepcopy(cinema)
        groups = [int(x) for x in input().split(" ")]

        #offline = Offline(cinema, groups)
        offlineMFFD = OfflineMFFD(cinema, groups)
        solved_cinema = offlineMFFD.solve()

        self._print_cinema(solved_cinema)
        print(str(cinema.get_amount_people_seated()) + ' people seated of ' + str(sum(offlineMFFD._get_group_sizes(groups))) + ' guests we recieved')


        offline = Offline(cinema2, groups)
        solved_cinema = offline.solve()

        self._print_cinema(solved_cinema)
        print(str(cinema2.get_amount_people_seated()) + ' people seated of ' + str(sum(offlineMFFD._get_group_sizes(groups))) + ' guests we recieved')

    def online(self) -> None:
        pass
