import copy
from typing import List

from cinematinator.types import Cinema


class OfflineMFFD:
    """
    This class models and solves a problem of the `offline` variation.
    """

    cinema: Cinema
    groups: List[int]

    def __init__(self, cinema: Cinema, groups: List[int]):
        self.cinema = cinema
        self.groups = groups

    def solve(self) -> Cinema:
        """
        Modified First Fit Decreasing aproximation
        """
        option1 = self._solveMFFD(self.cinema, self._get_group_sizes(self.groups))
        return option1

    def _get_group_sizes(self, groups: List[int]) -> List[int]:
        group_sizes = []
        for i in range(len(groups)):
            if groups[i] > 0:
                for _ in range(groups[i]):
                    group_sizes.append(i + 1)
        group_sizes.sort(reverse=True)
        return group_sizes

    def _solveMFFD(self, cinema: Cinema, groups: List[int]) -> Cinema:
        return self._solveFFD(self.cinema, groups)


    def _solveFFD(self, cinema: Cinema, groups: List[int]) -> Cinema: #First fit decreasing
        row_capacities = cinema.get_row_capacities()

        for group_size in groups:
            foundSeat = False
            print(group_size)
            for y in range(cinema.height):
                if foundSeat:
                    break
                if row_capacities[y] >= group_size:
                    for x in range(cinema.width):
                        if cinema.can_seat_group(x, y, group_size):
                            cinema.seat_group(x, y, group_size)
                            row_capacities[y] -= group_size
                            foundSeat = True
                            break

        return cinema
